import random

from faker import Faker
from src.models.endpoints.configs import config as endpoint_config

class HeadersManager:
    def __init__(self, locale="ru_RU"):
        self._faker = Faker(locale=locale)
        self._config = endpoint_config
        self._header_generators = self._config.header_generators

    def configure_generator(self, header, generator=None, values=None, fixed_value=None):
        if header not in self._header_generators:
            raise ValueError(f"Неизвестный заголовок: {header}")
        if fixed_value is not None:
            self._header_generators[header] = lambda: fixed_value
        elif values is not None:
            if not values:
                raise ValueError(f"Список значений для {header} не может быть пустым")
            self._header_generators[header] = lambda: random.choice(values)
        elif generator is not None:
            self._header_generators[header] = generator
        return self

    def get_endpoint_config(self, service, endpoint):
        return self._config.get_endpoint_config(service, endpoint)

    def generate_headers(self, service, endpoint, dynamic_values=None):
        dynamic_values = dynamic_values or {}
        config = self.get_endpoint_config(service, endpoint)
        headers = config.headers
        header_config = config.header_config

        for header, conf in header_config.items():
            self.configure_generator(
                header,
                generator=conf.get("generator"),
                values=conf.get("values"),
                fixed_value=conf.get("fixed_value")
            )

        if not headers:
            raise ValueError("Список заголовков не может быть пустым")
        unknown_headers = [header for header in headers if header not in self._header_generators]
        if unknown_headers:
            raise ValueError(f"Неизвестные заголовки: {unknown_headers}")

        result = {}
        for header in headers:
            if header in dynamic_values:
                result[header] = dynamic_values[header]
            else:
                result[header] = self._header_generators[header]()
        return result

    def get_headers_missing(self, service, endpoint, missing_header, dynamic_values=None):
        config = self.get_endpoint_config(service, endpoint)
        headers = config.headers
        header_config = config.header_config

        if missing_header not in headers:
            raise ValueError(f"Заголовок {missing_header} отсутствует в списке заголовков")

        for header, conf in header_config.items():
            self.configure_generator(
                header,
                generator=conf.get("generator"),
                values=conf.get("values"),
                fixed_value=conf.get("fixed_value")
            )

        dynamic_values = dynamic_values or {}
        result = {}
        for header in headers:
            if header != missing_header:
                if header in dynamic_values:
                    result[header] = dynamic_values[header]
                else:
                    result[header] = self._header_generators[header]()
        return result

    def __str__(self):
        return f"Headers: {list(self._header_generators.keys())}"


from functools import wraps
import time

stats = {}

def measure_response_time(endpoint_name, num_tests=1):
    def decorator(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            api_client = kwargs.get('api_client')
            if api_client is None:
                raise ValueError("api_client не найден в аргументах теста")

            original_make_request = api_client.make_request

            async def wrapped_make_request(*mr_args, **mr_kwargs):
                start_time = time.time()
                response = await original_make_request(*mr_args, **mr_kwargs)
                end_time = time.time()
                elapsed_time = end_time - start_time

                if endpoint_name not in stats:
                    stats[endpoint_name] = []
                stats[endpoint_name].append(elapsed_time)
                return response

            api_client.make_request = wrapped_make_request
            result = await func(*args, **kwargs)
            api_client.make_request = original_make_request
            return result
        return wrapper
    return decorator


class BColors:
    """
    A class used to represent color codes for console output.

    ...

    Attributes
    ----------
    HEADER : str
        Color code for header.
    OKBLUE : str
        Color code for blue text.
    OKCYAN : str
        Color code for cyan text.
    OKGREEN : str
        Color code for green text.
    WARNING : str
        Color code for warning text.
    FAIL : str
        Color code for fail text.
    ENDC : str
        Color code to reset color.
    BOLD : str
        Color code for bold text.
    UNDERLINE : str
        Color code for underlined text.
    """

    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


class MethodTypes:
    """
    A class used to represent HTTP method types.

    ...

    Attributes
    ----------
    type_list : list
        List of HTTP method types.
    """
    type_list = ['GET', 'POST', 'PUT', 'PATCH', 'DELETE']



from .logger import *
from .date import *
from .ascii import *
from .checking import *
#from .params_manager import *
#from .headers_manager import *
#from .response_timer import *
#from .compare_stands import *


import json

state1 = 'dev'
state2 = 'prod'

def compare_stands(file1, file2):
	with open(file1, 'r') as f1, open(file2, 'r') as f2:
		stats1 = json.load(f1)
		stats2 = json.load(f2)

	print(
		f"{'Endpoint':<20} {'Min (dev)':<10} {'Min (prod)':<10} {'Diff':<10} {'Avg (dev)':<10} {'Avg (prod)':<10} {'Diff':<10} {'Max (dev)':<10} {'Max (prod)':<10} {'Diff':<10}")
	print("-" * 100)

	for endpoint in set(stats1.keys()) | set(stats2.keys()):
		s1 = stats1.get(endpoint, {"min_time": 0, "max_time": 0, "avg_time": 0})
		s2 = stats2.get(endpoint, {"min_time": 0, "max_time": 0, "avg_time": 0})

		min_diff = s1["min_time"] - s2["min_time"]
		avg_diff = s1["avg_time"] - s2["avg_time"]
		max_diff = s1["max_time"] - s2["max_time"]

		print(
			f"{endpoint:<20} {s1['min_time']:<10.3f} {s2['min_time']:<10.3f} {min_diff:<10.3f} {s1['avg_time']:<10.3f} {s2['avg_time']:<10.3f} {avg_diff:<10.3f} {s1['max_time']:<10.3f} {s2['max_time']:<10.3f} {max_diff:<10.3f}")


if __name__ == "__main__":
	compare_stands("C:\\src\\aeroidea\\aerosite\\api_autotests\\response_times_dev.json", "C:\\src\\aeroidea\\aerosite\\api_autotests\\response_times_prod.json")


import json
from partest.utils import BColors


class ErrorDesc:
    """
        A class used to represent error descriptions.

        ...

        Attributes
        ----------
        codeExpected : int
            Expected status code.
        codeActual : int
            Actual status code.
        responseBody : str
            Response body.
        responseHeader : str
            Response header.
        requestHeader : str
            Request header.
        requestBody : str
            Request body.
        payloadElement : str
            Payload element.
        dataElement : str
            Data element.

        Methods
        -------
        status(codeExpected=200, codeActual=None, responseBody=None)
            Returns a string describing a status error.
        validate(validateData=None, validateModel=None, error=None)
            Returns a string describing a validation error.
        _ifelse()
            Returns a string describing an if-else error.
        element(payloadElement=None, dataElement=None, requestBody=None, responseBody=None)
            Returns a string describing an element error.
        __str__()
            Returns a string representation of the error.
        """

    def __init__(self):
        self.codeExpected = None
        self.codeActual = None
        self.responseBody = None
        self.responseHeader = None
        self.requestHeader = None
        self.requestBody = None
        self.payloadElement = None
        self.dataElement = None


    @classmethod
    def status(cls, codeExpected=200, codeActual=None, responseBody=None):
        """
        Returns a string describing a status error.

        Parameters
        ----------
        :param codeExpected: Expected status code.
        :param codeActual: Actual status code.
        :param responseBody: Response body.

        Returns
        -------
        :return: String describing a status error.
        """
        try:
            body = f'Response body: \n{json.dumps(responseBody.json(), indent=4, ensure_ascii=False)}\n'
        except:
            body = f'Response (non-JSON): {responseBody.text}\n'
        finally:
            desc = (f"\n\n---->\nОшибка! Пришел некорректный статус код!\n"
                    f"Ожидали код: {codeExpected} <\> Получили код: {codeActual}\n"
                    f"{body}\n<----\n\n")
        return desc

    @classmethod
    def validate(cls, validateData=None, validateModel=None, error=None):
        """
        Returns a string describing a validation error.

        Parameters
        ----------
        :param validateData: Validation data.
        :param validateModel: Validation model.
        :param error: Error.

        Returns
        -------
        :return: String describing a validation error.
        """
        try:
            return "\n\n---->\nОшибка валидации, объекты сравнения:", error, validateModel, '\n', validateData, "\nПодробнее в принт-логах.\n<----\n\n"
        except:
            return "Нет тела или модели"

    @classmethod
    def _ifelse(cls):
        """
        Returns a string describing an if-else error.

        Returns
        -------
        :return: String describing an if-else error.
        """
        try:
            return "\n\n---->\nНе выполнены условия для выполнения теста, падение.\n<----\n\n"
        except:
            return "Чёт пошло не так"

    @classmethod
    def element(cls, payloadElement=None, dataElement=None, requestBody=None, responseBody=None ):
        """
        Returns a string describing an element error.

        Parameters
        ----------
        :param payloadElement: Payload element.
        :param dataElement: Data element.
        :param requestBody: Request body.
        :param responseBody: Response body.

        Returns
        -------
        :return: String describing an element error.
        """
        try:
            if requestBody is not None and responseBody is not None:
                _resp_body = f'Response body: \n{json.dumps(responseBody.json(), indent=4, ensure_ascii=False)}\n'
                _req_body = f'Request body: \n{json.dumps(requestBody.json(), indent=4, ensure_ascii=False)}\n'
            else:
                _resp_body = ""
                _req_body = ""
        except:
            if requestBody is not None and responseBody is not None:
                _resp_body = f'Response (non-JSON): {responseBody.text}\n'
                _req_body = f'Request (non-JSON): {requestBody.text}\n'
            else:
                _resp_body = ""
                _req_body = ""
        finally:
            desc = (f"\n\n---->\nОшибка! Получили не то значение элемента что ожидали!\n"
                    f"Ожидали значение: {payloadElement} <\> Получили значение: {dataElement}\n"
                    f"{_resp_body}\n{_req_body}\n<----\n\n")
        return desc

    def __str__(self):
        """
        Returns a string representation of the error.

        Returns
        -------
        :return: String representation of the error.
        """
        return "Ответ не валиден"

class StatusCode:
    """
    A class used to represent status codes.

    ...

    Attributes
    ----------
    ok : int
        Status code for OK.
    bad_request : int
        Status code for bad request.
    not_allowed : int
        Status code for not allowed.
    forbidden : int
        Status code for forbidden.
    not_found : int
        Status code for not found.
    exception_400 : list
        List of status codes for exception 400.
    """
    ok = 200
    bad_request = 400
    not_allowed = 405
    forbidden = 403
    not_found = 404
    exception_400 = [200, 400]


class PydanticResponseError:
    @staticmethod
    def print_error(e):
        print(BColors.WARNING + "\n__________<ReportValidate>__________" + BColors.ENDC)
        print(BColors.BOLD + "Ошибка валидации, тип:" + BColors.ENDC,
              BColors.FAIL + repr(e.errors()[0]['type']),
              ":", repr(e.errors()[0]['msg']) + BColors.ENDC)
        print(BColors.BOLD + "Проблемный ключ:" + BColors.ENDC, repr(e.errors()[0]['loc']))
        print(BColors.BOLD + "Входящее значение:" + BColors.ENDC, repr(e.errors()[0]['input']))
        print(BColors.BOLD + "Полный текст ошибки:" + BColors.ENDC, repr(e.errors()))
        print(BColors.WARNING + "__________</ReportValidate>__________" + BColors.ENDC)